from django.urls import reverse
from accounts import forms

from django.views import generic
from django.contrib.auth import login, get_user_model
from django.shortcuts import redirect
from django.core import exceptions


class RegisterView(generic.CreateView):
    model = get_user_model()
    template_name = 'auth/register.html'
    form_class = forms.RegisterForm
    FORM_TYPE_FORMS = {
        'primary': forms.PrimaryRegisterForm,
        'secondary': forms.SecondaryRegisterForm,
    }
    
    def dispatch(self, request, *args, **kwargs):
        self.form_type = self.request.GET.get('form_type', 'primary')

        if self.form_type.lower() not in self.FORM_TYPE_FORMS:
            raise exceptions.BadRequest

        return super().dispatch(request, *args, **kwargs)
    
    def get_form_class(self):
        return self.FORM_TYPE_FORMS.get(self.form_type, self.form_class)
    
    def get_initial(self):
        initial = super().get_initial()
        
        if self.request.method == 'GET' and self.form_type == 'secondary':
            initial |= {'id': self.request.GET.get('id')}
        
        return initial

    def form_valid(self, form):
        if self.form_type == 'primary':
            user = form.save()

            return redirect(reverse('register') + f'?form_type=secondary&id={user.id}')

        elif self.form_type == 'secondary':
            user = get_user_model().objects.get(id=form.cleaned_data['id'])

            user.is_active = True
            user.set_password(form.cleaned_data['password1'])
            user.save()

            login(self.request, user)
            return redirect(self.get_success_url())
    
    def get_success_url(self):
        return self.request.POST.get(
            'next',
            self.request.GET.get('next', 'index')
        )
